#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################

#######
def get_require_modules(given_file):
    """
    Get main python requirements modules
    """
    with open(given_file, 'r') as f:
        myModules = [line.strip().split(',')[0] for line in f]
    
    return myModules

#######
def get_version(version_file):
    """
    Original code: PhiSpy setup.py 
    https://github.com/linsalrob/PhiSpy/blob/master/setup.py
    """
    with open(version_file, 'r') as f:
        v = f.readline().strip()
    return v


#!/usr/bin/env python3
#############################################################
## Jose F. Sanchez, Marta Lopez & Lauro Sumoy              ##
## Copyright (C):2012 Davud Li Wei - https://weililab.org/ ##
##               2019-2021 Lauro Sumoy Lab, IGTP, Spain    ##
#############################################################
"""
gtf2bed.py converts GTF file to BED file.
Usage: gtf2bed.py {OPTIONS} [.GTF file]
History
    Nov.5th 2012:
        1. Allow conversion from general GTF files (instead of only Cufflinks supports).
        2. If multiple identical transcript_id exist, transcript_id will be appended a string like "_DUP#" to separate.

    Nov.9th 2021: Modified by JFSanchezherrero
        1. Reformat code to use as a function
        2. Add additional details
        3. Simplify

Copyrigth: http://alumni.cs.ucr.edu/~liw/scripts.html
BED Format details: https://genome.ucsc.edu/FAQ/FAQformat.html#format1
"""

import sys
import os
import re
from termcolor import colored

import HCGB.functions.aesthetics_functions as HCGB_aes
import HCGB.functions.files_functions as HCGB_files
import HCGB.functions.time_functions as HCGB_time

allids = {}

############################################################
def parse_GTF_call(gtf_file, out_file, debug=False):

    ## debug messaging    
    if debug:
        print()
        HCGB_aes.debug_message("********************** parse_GTF_call ********************** ")
        HCGB_aes.debug_message("Checking if GTF has been previously converted to BED", "yellow")
    
    path_given=os.path.dirname(out_file)
    name = HCGB_files.get_file_name(out_file)
    gtf_file = os.path.abspath(gtf_file)
        
    ## get absolute path and name
    name_file = HCGB_files.get_path_name(gtf_file, path_given, 
                              name, debug=debug) + ".bed"
                              
    if debug:
        HCGB_aes.debug_message("gtf_file: " + gtf_file, "yellow")
        HCGB_aes.debug_message("name: " + name, "yellow")
        HCGB_aes.debug_message("out_file: " + out_file, "yellow")
        HCGB_aes.debug_message("name_file: " + name_file, "yellow")
                              

    filename_stamp = os.path.join(path_given, '.' + name + "_GTF_convertion_success")
    if HCGB_files.is_non_zero_file(filename_stamp):
        if HCGB_files.is_non_zero_file(out_file):
            ## bed files exists
            if debug:
                HCGB_aes.debug_message("Timestamp exists: .split_GTF_success ", "yellow")
                print()
    
            print (colored("\tA previous command generated results on: %s [%s]" %(HCGB_time.read_time_stamp(filename_stamp)
                                                                                  , 'convert GTF -> BED'), 'yellow'))
            
            return(out_file)
    
    ## File is not processed or not finished
    parse_GTF(gtf_file, out_file, debug=debug)
    
    ## print time stamp
    HCGB_time.print_time_stamp(filename_stamp)
            
    if debug:
        HCGB_aes.debug_message("********************** parse_GTF_call ********************** ")
    
    return (out_file)


############################################################
def savebedline(estart, eend, field, nline, debug):
    """
    
    est=int(field[3])
    eed=int(field[4])
    estart +=[est]
    eend +=[eed]
    
    :param estart: Contains feature coordinates for each exon start (+) or end (-).
    :param eend: Contains feature coordinates for each exon start (-) or end (+).
    :param field: GTF line splitted. Contains 9 fields.
    :param nline: Number of line in the GTF file
    :param debug: True/False for debugging messages
    
    :type estart: list
    :type eend: list
    :type field: list
    :type nline: int
    :type debug: bool
    
    :returns: String to write in BED format.
    """
    
    ## debug messages    
    if debug:
        HCGB_aes.debug_message("GTF line: ", "yellow")
        print(field)
        
    ## ---------------------------------
    # use regular expression to get transcript_id, gene_id and expression level
    ## ---------------------------------
    #geneid=re.findall(r'gene_id +\"([\w\.]+)\";',field[8])
    #transid=re.findall(r'transcript_id +\"([\w\.]+)\";',field[8])
    #fpkmval=re.findall(r'FPKM +\"([\d\.]+)\";',field[8])
    #biotype=re.findall(r'transcript_biotype +\"([\w\.]+)\";',field[8])
    
    geneid=re.findall(r'gene_id +\"(.+)[\";]+.*',field[8])
    transid=re.findall(r'transcript_id +\"(.+)[\";]+.*',field[8])
    fpkmval=re.findall(r'FPKM +\"(.+)[\";]+.*',field[8])
    biotype=re.findall(r'transcript_biotype +\"(.+)[\";]+.*',field[8])
    ## add others if required: e.g. transcript_biotype
    
    ## ---------------------------------
    ## Get Gene ID
    ## ---------------------------------
    if len(geneid)==0:
        print('Warning: no gene_id field in line ' + str(nline))
        gene_id="none"
    else:
        gene_id=geneid[0].split('\"')[0]
    
    ## ---------------------------------
    ## Get transcript biotype
    ## ---------------------------------
    if len(biotype)==0:
        print('Warning: no transcript_biotype field in line ' + str(nline))
        transcript_biotype="none"
    else:
        transcript_biotype=biotype[0].split('\"')[0]

    ## ---------------------------------
    ## FPKM field
    ## ---------------------------------
    fpkmint=100
    if len(fpkmval)>0: # Warning: no FPKM field
        fpkmval=fpkmval[0].split('\"')[0]
        fpkmint=round(float(fpkmval))

    ## ---------------------------------
    ## Get transcript ID
    ## ---------------------------------
    if len(transid)==0: # Warning: no transcript_id field
        transid='Trans_'+str(nline) ## set a new name for transcript if missing
        print('Warning: no transcript_id field in line ' + str(nline))
        print('Warning: Generate new: ' + transid)
    else:
        transid=transid[0].split('\"')[0]
    
    ## Previous transcript ID
    if transid in allids.keys():
        transid2=transid+'_DUP'+str(allids[transid])
        allids[transid]=allids[transid]+1
        transid=transid2;
    ## New transcript ID
    else:
        allids[transid]=1
    
    ## ---------------------------------
    ## Get exon start and lengths
    ## ---------------------------------
    if (field[6]=="+"):
    
        seglen=[eend[i]-estart[i]+1 for i in range(len(estart))]
        estp=estart[0]-1
        eedp=eend[-1]
        segstart=[estart[i]-estart[0] for i in range(len(estart))]
    
    elif (field[6]=="-"):
        seglen=[eend[i]-estart[i]+1 for i in range(len(estart))]
        estp=estart[0]-1
        eedp=eend[-1]
    
        segstart=[estart[0]-estart[i] for i in range(len(estart))]
        if len(estart)!=1: ## if a single exon by gene, it is correct. Otherwise, we need to "transpose". 
            estp=eend[-1]
            eedp=estart[0]-1

    strl=str(seglen[0])
    for i in range(1,len(seglen)):
        strl+=','+str(seglen[i])

    strs=str(segstart[0])
    for i in range(1,len(segstart)):
        strs+=','+str(segstart[i])

    ## debug messages    
    if debug:
        HCGB_aes.debug_message("estart: " + str(estart), "yellow")
        HCGB_aes.debug_message("seglen: " + str(seglen), "yellow")
        HCGB_aes.debug_message("segstart: " + str(segstart), "yellow")
        HCGB_aes.debug_message("strl: " + str(strl), "yellow")
        HCGB_aes.debug_message("strs: " + str(strs), "yellow")
    
    ## ---------------------------------
    ## Save data information
    ## ---------------------------------
    ###            #"chr"            "start"            "end"         "transcript_id"            "fpkm"             "strand"      "thickstart"        "thickend"             "RGB"             "estart"               "strs"        "strl"     "gene_id"         "transcript_biotype"
    string2write = field[0] + '\t' + str(estp) + '\t' + str(eedp) + '\t' + transid + '\t' +  str(fpkmint)+ '\t' +  field[6]+ '\t' +  str(estp) + '\t' + str(eedp) + '\t' + "255,0,0"+ '\t' + str(len(estart)) + '\t' + strs+ '\t' + strl+ '\t' + gene_id+ '\t' + transcript_biotype

    ## debug messages    
    if debug:        
        HCGB_aes.debug_message("data BED format: " + transid, "yellow")
        print(string2write)

    return (string2write)
    
############################################################
def parse_GTF(gtf_file, out_file, debug):
    
    original_stdout = sys.stdout # Save a reference to the original standard output

    ## Start the parsing of GTF
    ## Init variables 
    estart=[]
    eend=[]
    nline=0 # read lines one to one
    prevfield=[]
    prevtransid=''

    if debug:
        print("")
        HCGB_aes.debug_message("*************")
        
    with open(out_file, 'w') as f:
        sys.stdout = f # Change the standard output to the file we created.
        ## Loop through big GTF file
        for lines in open(gtf_file):
            field=lines.strip().split('\t')
            ## count lines
            nline=nline+1
            
            ## debug messages
            if debug:
                HCGB_aes.debug_message("nline: " + str(nline), "yellow")
                print(lines)
                

                
            ## skip comment lines
            if field[0].startswith("#"): ## Comment line: skipping
                continue
                    
            if len(field)<9:
                print('Error: the GTF should has at least 9 fields at line ')
                continue
    
            if field[1]!='Cufflinks':
                pass
                #print('Warning: the second field is expected to be \'Cufflinks\' at line '+str(nline),file=sys.stderr)
            
            if field[2]!='exon' and field[2] !='transcript':
                if debug:        
                    HCGB_aes.debug_message("Line: ", "yellow")
                    print(lines, file=sys.stderr)
                    HCGB_aes.debug_message("nline: " + str(nline), "yellow")
                    print('Error: the third filed is expected to be exon or transcript')
                    
                continue
            
            ## ---------------------------------
            # use regular expression to get transcript_id
            #transid=re.findall(r'transcript_id +\"([\w\.]+)\";',field[8])
            transid=re.findall(r'transcript_id +\"(.+)[\";]+.*',field[8])
            if len(transid)>0:
                transid=transid[0].split('\"')[0]
            else:
                transid=''
            
            ## debug messages
            if debug:
                HCGB_aes.debug_message("prevtransid: " + str(prevtransid))
                HCGB_aes.debug_message("transid: " + transid)
            
            ## when changes, save previous field information                
            if field[2]=='transcript' or (prevtransid != '' and transid!='' and transid != prevtransid):
                #print('prev:'+prevtransid+', current:'+transid)
                # A new transcript record, write
                if len(estart)!=0:
                    ## debug messages    
                    if debug:        
                        HCGB_aes.debug_message("savebedline call")
                        HCGB_aes.debug_message("estart: " + str(estart))
                        HCGB_aes.debug_message("eend: " + str(eend))
                        HCGB_aes.debug_message("prevfield: " + str(prevfield))
                    
                    ## save record in bed format
                    strin2print = savebedline(estart, eend, prevfield, nline, debug)
                    print(strin2print)
                    
                    ## debug messages
                    if debug:        
                        HCGB_aes.debug_message("*************")
                        print("")
                    
                # Reset
                estart=[]
                eend=[]
                        
            ## ---------------------------------
            prevfield=field
            prevtransid=transid
            if field[2]=='exon':
                try:  
                    est=int(field[3])
                    eed=int(field[4])
                    estart +=[est]
                    eend +=[eed]
                except ValueError:
                    print('Error: non-number fields at line '+str(nline))
      
        #############################        
        # the last record
        #############################
        if len(estart)!=0:
            ## debug messages    
            if debug:        
                HCGB_aes.debug_message("savebedline call")
                HCGB_aes.debug_message("transid: " + transid)
                HCGB_aes.debug_message("estart: " + str(estart))
                HCGB_aes.debug_message("eend: " + str(eend))
                HCGB_aes.debug_message("prevfield: " + str(prevfield))
                HCGB_aes.debug_message("prevtransid: " + str(prevtransid))

            ## save record in bed format
            strin2print = savebedline(estart, eend, field, nline, debug)
            print(strin2print)
    
    # Reset the standard output to its original
    sys.stdout = original_stdout  

    ## ---------------------------------
    ## return when finished
    return()

############################################################
def main():
    ## this code runs when call as a single script
    if len(sys.argv)<2:
        print('This script converts .GTF into .BED annotations.\n')
        print('Usage: gtf2bed [.GTF file] [OUT bed]\n')
        print('\nNote:')
        print('1\tOnly "exon" and "transcript" are recognized in the feature field (3rd field).')
        print('2\tIn the attribute list of .GTF file, the script tries to find "gene_id", "transcript_id" and "FPKM" attribute, and convert them as name and score field in .BED file.') 
        
        print('Author: Wei Li (li.david.wei AT gmail.com)')
        sys.exit()

    ## get output file
    out_file=""
    if len(sys.argv)>=3:
        out_file = sys.argv[2]
    else:
        out_file = "example.bed"
    
    out_file = os.path.abspath(out_file)
    
    print("+ Converting GTF to bed...")
    
    ## parse 
    parse_GTF_call(sys.argv[1], out_file, False)

    ## final

############################################################
if __name__== "__main__":
    main()



#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Shared functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
With different purposes:
    - Print time stamps
  
    - Create system calls

    - Manage/list/arrange files/folders

    - Aesthetics

    - Manage fasta files

    - Other miscellaneous functions
"""
## useful imports
import sys
import os
import pandas as pd
import urllib.request
import json

from HCGB import functions

###################################
##     General main functions    ##
###################################

#################
def print_all_pandaDF(pd_df):
    pd.set_option('display.max_colwidth', None)
    pd.set_option('display.max_columns', None)
    print (pd_df)

#################
def printList2file(fileGiven, listGiven):
    """Prints list given in the output file provided. One item per row."""
    file_hd = open(fileGiven, 'w')
    file_hd.write("\n".join(listGiven))
    file_hd.close()

#################
def printDict2file(fileGiven, dictGiven, split_char):
    """Prints dictionary given in the output file provided. One item per row."""
    with open(fileGiven, 'w') as file_hd:
        for key in dictGiven.keys():
            file_hd.write("%s%s%s\n"%(key, split_char, dictGiven[key]))
    file_hd.close()

#################
def readList_fromFile(fileGiven):
    """Reads list from the input file provided. One item per row."""
    # open file and read content into list
    lineList = [line.rstrip('\n') for line in open(fileGiven)]
    return (lineList)

###########
def retrieve_matching_files(folder, string, debug=False, starts=False):
    """Lists folder path provided and given a string to search, returns all files ending with the given string"""
    my_all_list = get_fullpath_list(folder)
    if starts:
        matching = [s for s in my_all_list if s.startswith(string)]
    else:
        matching = [s for s in my_all_list if s.endswith(string)]
        
        
    return (matching)

##########
def file2dictionary(file2read, split_char):
    """Read file and generate a dictionary"""
    d = {}
    with open(file2read) as f:
        for line in f:
            line = line.rstrip('\n')
            (key, val) = line.split(split_char)
            d[key] = val

    return(d)

###############
def file2dataframe(file2read, names):
    ## TODO: check if duplicated with get_data function
    """Read csv file into pandas dataframe"""
    d = pd.read_csv(file2read, comment="#", names=names)
    return(d)

#################
def get_fullpath_list(dir_given, debug=False):
    """Retrieve full absolute path for the files within a directory specified.

    :param dir_given: Directory to retrieve files
    :type dir_given: string

    :returns: List of absolute path files.
    """
    return_path = []
    for root, dirs, files in os.walk(dir_given):
        for f in files:
            return_path.append(os.path.join(root,f))

    ## returns list of files
    return return_path

###############
def get_data(ID_file, SEP, options):    
    if options == 'index_col=0':
        data = pd.read_csv(ID_file, sep=SEP, index_col=0)
        return(data)
    elif options == 'header=None':
        data = pd.read_csv(ID_file, sep=SEP, header=None)
        return(data)    
    else:
        data = pd.read_csv(ID_file, sep=SEP)
        return(data)
    ## fix for another example if any
    
###############
def get_number_lines(input_file):    
    with open(input_file) as foo:
        lines = len(foo.readlines())
    foo.close()
    return (lines)

###############
def get_info_file(input_file):    
    with open(input_file) as f:
        lines = f.read().splitlines() 
    f.close()
    return (lines)
          
#################
def optimize_threads(total, samples):
    cpu = int(int(total)/int(samples))
    
    if (cpu==0): ## 5 availabe cpus for 10 samples == 0 cpu
        cpu = 1
    
    return(cpu)

#################
def parse_sublist(lst, ind): 
    ## extract elemnts of the list
    ## Original Code: https://www.geeksforgeeks.org/python-get-first-element-of-each-sublist/
    return [item[ind] for item in lst]

##################
def decode(x):
    """
    Python decode string method

    It converts bytes to string.

    :param x: String of text to decode
    :type x: string
    :returns: Text decoded

    .. attention:: Be aware of Copyright

        The code implemented here was retrieved and modified from ARIBA (https://github.com/sanger-pathogens/ariba)

        Give them credit accordingly.
    """
    try:
        s = x.decode()
        return s
    except:
        return x

##################
def urllib_request(folder, url_string, file_name, debug):
    
    file_name_unzipped="n.a."
    zipped=False
    
    ## add more compression types if necessary
    if file_name.endswith('gz'):
        zipped=True
        file_name_unzipped = file_name.split('.gz')[0]
    else:
        file_name_unzipped = file_name    
    ### debugging messages
    if debug:
        print ("folder: ", folder)
        print ("url_string: ",url_string)
        print ("file_name:", file_name)
        print ("Zip: ", zipped)
        print ("file_name_unzipped: ", file_name_unzipped)
        
    ##
    url_string = url_string + file_name
        
    
    ## timestamp
    filename_stamp = folder + '/.success'
    file_path_name = os.path.join(folder, file_name)
    file_path_name_unzipped = os.path.join(folder, file_name_unzipped)
    
    ## check if previously exists
    if os.path.exists(file_path_name) or os.path.exists(file_path_name_unzipped):
        ## debug message
        if debug:
            print ("## Debug: File %s exists in folder" %file_name)
    
        ## check if previously timestamp generated
        if os.path.isfile(filename_stamp):
            stamp = functions.time_functions.read_time_stamp(filename_stamp)
            print ("\tFile (%s) Previously downloaded in: %s" %(file_name, stamp))
            return(file_path_name_unzipped)

    ## debug message
    if debug:
        print ("## Debug: Download file: ", file_name, ":: ", url_string)
    
    # downloads
    urllib.request.urlretrieve(url_string, file_path_name )
    
    ## check if correctly download
    if os.path.exists(file_path_name):
        functions.time_functions.print_time_stamp(filename_stamp)
    else:
        sys.exit('Could not download file %s (%s)' %(file_name, url_string))

    ## extract if zipped
    if (zipped):
        functions.files_functions.extract(file_path_name, out=folder)
        return (file_path_name_unzipped)
        
    else:
        return(file_path_name)

###################
def create_excel_file(file_name, dict_data, SEP=",", options="index_col=0", debug=False):
    '''
    :param file_name: Absolute file path name for the file to write in xlsx format.
    :param dict_data: Dictionary containing as key the name for the tab and as a value an absolute file path to dump in each tab.
    '''
    
    ## write to excel
    writer_Excel = pd.ExcelWriter(file_name, engine='xlsxwriter') ## open excel handle
    
    ## loop over dictionary and print dataframe
    for name, each_data in dict_data.items():
        if debug:
            print ("Name: ", name)
            print ("Data: ", each_data)
            
        each_df = get_data(each_data, SEP, options)
        each_df.to_excel(writer_Excel, sheet_name=name) ## write excel handle
            
    writer_Excel.save() ## close excel handle

######################################
def read_json_file(file, debug=False):
    """Read information in json format"""

    if debug:
        functions.aesthetics_functions.debug_message("Read JSON information:", 'yellow')
        functions.aesthetics_functions.debug_message("file: " + file, 'yellow')
    
    with open(file) as json_file:
        data=json.load(json_file)

    if debug:
        print(data)
    
    ## return json
    return (data)
    


#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain    ##
##########################################################
"""
Provides configuration for the pipeline.
"""
## useful imports
import os
import re
import subprocess
from termcolor import colored
from distutils.version import LooseVersion
import pkg_resources

## import my modules
from HCGB import functions
from HCGB.config import extern_progs

################
## Software
################

##################
def get_exe(prog, Debug=False, Return_Version=False):
    """Return absolute path of the executable program requested.

    Given a program name it returns its executable to be called. It has to fulfilled a minimum version specified.

    :param prog: Software name
    :type prog: string
    :returns: Absolute path for the executable requested
    :warning: if no executable available in system ``$PATH`` or not fulfilling the expected version.

    .. attention:: Be aware of Copyright

        The code implemented here was retrieved and modified from ARIBA (https://github.com/sanger-pathogens/ariba)

        Give them credit accordingly.

    """
    exe = ""
    if prog in os.environ: 
        exe = os.environ[env_var] ## python environent variables
    else:
        exe = extern_progs.return_defatult_soft(prog) ## install in the system

    ## get paths
    exe_path_tmp = my_which(exe)

    ## debug message
    if (Debug):
        print(colored("** Debug: exe: %s" %exe,'yellow'))
        print(colored("** Debug: exe_path_tmp: %s" %exe_path_tmp,'yellow'))

    ## get min_version
    min_version = extern_progs.return_min_version_soft(prog)
    
    ## debug message
    if (Debug):
        print(colored("** Debug: min_version: %s" %min_version,'yellow'))

    ## return if not available
    ## no min version available
    if min_version == 'na':
        if exe_path_tmp:
            if (Return_Version):
                return (exe_path_tmp[0], '') ## return first item
            else:
                return (exe_path_tmp[0]) ## return first item
    
    ## not installed in path
    if exe_path_tmp is None or len(exe_path_tmp) == 0:
        if (Return_Version):
            print(colored("\n**ERROR: Software %s could not be found." % prog,'red'))
            exit()
            return('ERROR', 'n.a.')
        else:
            print(colored("\n**ERROR: Software %s could not be found." % prog,'red'))
            exit()
            return('ERROR')
        

    ## Loop for all possibilities
    for p in exe_path_tmp:
        prog_ver = get_version(prog, p, Debug=Debug)

        if (Debug):
            print (colored("** Debug: Software: %s\nPath: %s\nVersion: %s" %(prog, p, prog_ver), 'yellow'))

        if (prog_ver == 'n.a.'):
            continue

        if LooseVersion(prog_ver) >= LooseVersion(min_version):
            if (Return_Version):
                return (p, prog_ver)
            else:
                return (p)


    print(colored("\n**ERROR: Software %s version smaller than minimum version expected %s." %(prog,min_version),'red'))
    exit()

################
def access_check(fn, mode=os.F_OK | os.X_OK):
    """Check executable permission

    This function checks whether a given path is a folder or file and if it is 
    executable and accessible. It also works if a java jar file provided.

    :param fn: Absolute path file
    :param mode: Value to pass as the mode parameter of access()

    :type fn: string
    :type mode: string

    `mode` defaults to:

        - os.F_OK: Value to pass as the mode parameter of access() to test the existence of path.

        - os.X_OK: Value to include in the mode parameter of access() to determine if path can be executed.

    .. attention:: Be aware of Copyright

        The code implemented here was retrieved and modified from shutil (https://github.com/python/cpython/blob/master/Lib/shutil.py).

        Give them credit accordingly.

        We modified the code to work if java jar files provided.
    """
    ## the original code belongs to shutil, slightly modified here
    # https://github.com/python/cpython/blob/master/Lib/shutil.py

    #if os.path.isdir(fn):
    #    return False

    if os.path.exists(fn):
        if fn.endswith('.jar'):
            return True

        if os.access(fn, mode):
            return True

#################
def my_which(cmd):
    """Return the absolute path to the executable

    Given a command return the absolute path(s), if any.

    :param cmd: Software command name 
    :returns: List of absolute paths(s) of the given command.

    .. attention:: Be aware of Copyright

        The code implemented here was retrieved and modified from shutil (https://github.com/python/cpython/blob/master/Lib/shutil.py).

        Give them credit accordingly.

        We modified the code to return multiple paths in a list if available different installed binaries in $PATH.

    """
    # If we're given a path with a directory part, look it up directly rather
    # than referring to PATH directories. This includes checking relative to the
    # current directory, e.g. ./script
    if os.path.dirname(cmd):
        if access_check(cmd):
            return cmd
        return None

    use_bytes = isinstance(cmd, bytes)

    path=None

    if path is None:
        path = os.environ.get("PATH", None)

    if path is None:
        try:
            path = os.confstr("CS_PATH")
        except (AttributeError, ValueError):
            # os.confstr() or CS_PATH is not available
            path = os.defpath
        # bpo-35755: Don't use os.defpath if the PATH environment variable is
        # set to an empty string

    # PATH='' doesn't match, whereas PATH=':' looks in the current directory
    if not path:
        return None

    if use_bytes:
        path = os.fsencode(path)
        path = path.split(os.fsencode(os.pathsep))
    else:
        path = os.fsdecode(path)
        path = path.split(os.pathsep)

    # On other platforms you don't have things like PATHEXT to tell you
    # what file suffixes are executable, so just pass on cmd as-is.
    files = [cmd]

    return_paths = [] ## modification
    seen = set()
    for dir in path:
        normdir = os.path.normcase(dir)
        #print ("Normdir: ", normdir)
        if not normdir in seen:
            seen.add(normdir)
            for thefile in files:
                name = os.path.join(dir, thefile)
                #print ("Name: ", name)
                if access_check(name):
                    ## return (name) ## previously, it would only return the first item
                    return_paths.append(name) ## modification

    if (len(return_paths) >= 1):
        return return_paths
    else:
        return None

##################
def get_version(prog, path, Debug=False):
    """Get version of software

    Given a program name and expected path, tries to determine its version.

    :param prog: Program name
    :param path: Absolute path
    :param Debug: True/False

    :type prog: string
    :type path: string 
    :type Debug: bool

    :returns: String containing version. Returns NA message if no found and raises attention error message.

    .. attention:: Be aware of Copyright

        The code implemented here was retrieved and modified from ARIBA (https://github.com/sanger-pathogens/ariba)

        Give them credit accordingly.
    """

    ## read dependencies information
    dependencies_pd = extern_progs.read_dependencies()

    ## get information for prog
    regex = re.compile(dependencies_pd.loc[prog, 'get_version'])
    args = dependencies_pd.loc[prog, 'version_cmd']
    cmd = path + ' ' + args

    ## debug messages
    if (Debug):
        print(colored("** Debug: regex: %s" %regex,'yellow'))
        print(colored("** Debug: args: %s" %args, 'yellow'))

    if prog == 'sRNAbench' or prog == "miraligner":
        java_bin = get_exe('java', Debug=Debug)
        java_jar = java_bin + ' -jar ' + path + ' ' + args
        cmd_output = subprocess.Popen(java_jar, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE).communicate()
    else:
        cmd_output = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()

    ## decode command
    cmd_output = functions.main_functions.decode(cmd_output[0]).split('\n')[:-1] + functions.main_functions.decode(cmd_output[1]).split('\n')[:-1]

    ## debug messages
    if (Debug):
        print(colored("** Debug: cmd_output:\n %s" %cmd_output,'yellow'))

    ## retrieve version information
    for line in cmd_output:
        hits = regex.search(line)
        if (Debug):
            print (hits)
        if hits:
            return hits.group(1)

    if Debug:
        print (colored('Attention: I tried to get the version of ' + prog + ' with: "' + cmd + '" and the output didn\'t match this regular expression: "' + regex.pattern + '"', 'red'))

    return("n.a.")

def check_dependencies(Debug):
    """
    Check if available the different software required for ``XICRA`` execution.
    
    Using the function :func:`XICRA.config.extern_progs.read_dependencies` the 
    information for all the dependencies is retrieved from file :file:`XICRA/config/software/dependencies.csv`.
    
    For each software, the function :func:`XICRA.config.set_config.get_exe` retrieves
    whether it is installed in the system or not and its version and it is check using 
    :func:`XICRA.config.set_config.check_install`. 
    
    :param Debug: True/False for debugging messages 
    :type Debug: boolean
    
    :returns: Print messages and information    
    """
    
    ## read dependencies information
    dependencies_pd = extern_progs.read_dependencies()

    for soft, row in dependencies_pd.iterrows():
        soft_name = row['soft_name']
        min_version = row['min_version']
        
        ## debug messages
        if (Debug):
            print ("Software:", soft)
            print ("Soft name:", soft_name)
            print ("Min_Version:", min_version)
        
        ## get executable
        (soft_path, installed) = get_exe(soft, Debug=Debug, Return_Version=True)
        
        ## debug messages
        if (Debug):
            print ("Soft Path: ", soft_path)
            print ("Version installed:", installed)
            
        ## check if installed
        if (min_version == 'na'):
            message = check_install_module('ok', soft_name, min_version, 'Software')
        else:
            message = check_install_module(installed, soft_name, min_version, 'Software')

        if (message == 'OK'):
            continue
        else:
            print ("+ Please install manually software: ", soft_name, " to continue with XICRA\n\n")
    
################
## Python
################
def get_python_packages(Debug):
    """
    Retrieves the version of the python packages installed in the system.

    It retrieves the dependencies name conversion from file :file:`XICRA/config/python/module_dependencies.csv`
    using function :func:`XICRA.config.extern_progs.file_list` and :func:`XICRA.scripts.functions.main_functions.get_data`.
    For each module it retrieves the package version installed in the system using 
    :func:`XICRA.config.set_config.check_package_version`.    

    :returns: Dictionary containing for each python module (key) the installed version (value).
    """

    ## get import names for packages:
    ## some modules do not have the same name when install from pip and called from import
    file_module_dependecies = extern_progs.file_list("python_requirements")
    module_dependencies = functions.main_functions.file2dictionary(file_module_dependecies, ',')

    my_packages_installed = {}
    for each in module_dependencies:
        module_name = module_dependencies[each]
        installed = check_package_version(each, Debug) ## check version installed in system
        my_packages_installed[each] = installed

    return (my_packages_installed)

##################
def check_python_packages(Debug):
    """
    This functions checks whether the packages installed in the system fulfilled the 
    minimum version specified in the configuration folder. 

    It uses function :func:`XICRA.config.set_config.get_python packages` to
    retrieve the version of the python packages installed in the system. Then it uses
    :func:`XICRA.config.extern_progs.min_python_module_version` to retrieve the minimum
    version specified. It compares them using function :func:`XICRA.config.set_config.check_install_module`.

    :param Debug: True/False for debugging messages
    :type Debug: boolean

    :returns: Print messages if packages are installed.
    """
    ## get python packages installed
    my_packages_installed = get_python_packages(Debug)

    ## debug messages
    if (Debug):
        print ("my_packages_installed :: ")
        print (my_packages_installed)

    ## min versions for packages
    my_packages_requirements = extern_progs.min_python_module_version()

    ## debug messages
    if (Debug):
        print ("my_packages_requirements")
        print (my_packages_requirements)

    ## check each package
    for each in my_packages_requirements:
        ## get min version
        min_version = my_packages_requirements[each]

        ## get version installed in system
        installed = my_packages_installed[each] 

        ## debug messages
        if (Debug):
            print ("Module:", each)
            print ("Min_Version:", min_version)
            print ("Version installed:", installed)
            
        ## check if installed
        message = check_install_module(installed, each, min_version, 'Module')

        if (message == 'OK'):
            continue
        else:
            print ("+ Please install manually package: ", each, " to continue with XICRA\n\n")
            #print ("pip install %s" %each)

################
def check_package_version(package, Debug):
    """
    Retrieve python package version installed

    This is a modification of the original code from ARIBA (https://github.com/sanger-pathogens/ariba). 
    It basically uses pkg_resources.get_distribution(), pkg_resources.resource_filename() or imports module
    and retrieves version from __version__ variable.

    :param package: Python package name 
    :param Debug: True/False for debugging messages

    :type package: string
    :type Debug: boolean

    :returns: Version retrieved

    .. attention:: Be aware of Copyright

        The code implemented here was retrieved and modified from ARIBA (https://github.com/sanger-pathogens/ariba)

        Give them credit accordingly.
    """

    try:
        version = pkg_resources.get_distribution(package).version
        if (Debug):
            print ("Method: pkg_resources.get_distribution(package).version")
    except:
        try:
            exec('import ' + package)
            version = eval(package + '.__version__')
            if (Debug):
                print ("Method: exec('import ' + package); version = eval(package + '.__version__')")
        except:
            try:
                if (Debug):
                    print ("Method: pkg_resources.resource_filename(package, 'version.py')")
                version = pkg_resources.resource_filename(package, 'version.py')
            except:
                version = 'n.a.'
    if (Debug):
        print ('Package:', package)
        print ('Version:', version)
    return(version)
################

################
## R
################
def get_R_packages():
    dep_file = os.path.abspath(os.path.join(os.path.dirname( __file__ ), 'R', 'R_dependencies.csv'))
    dep_file_data = functions.main_functions.get_data(dep_file, ',', 'index_col=0')
    return (dep_file_data)

################
def check_R_packages(Debug):
    
    packages = get_R_packages()
    check_install_system = os.path.abspath(os.path.join(os.path.dirname( __file__ ), 'R', 'check_install_system.R'))
    R_script_exe = get_exe('Rscript')
    
    for index,row in packages.iterrows():
        ## debugging messages
        if Debug:
            print ('\n+ Check package: ', index)
            print('+ Source: ', row['source'])
        
        cmd_check = R_script_exe + ' ' + check_install_system + ' -l ' + index
        code = functions.system_call_functions.system_call(cmd_check, message=False, returned=False)
        if (code=='OK'):
            check_install_module('1', index, '0', 'package')
        else:
            check_install_module('0', index, '1', 'System package')

            ## check if installed in path
            cmd_check_path = R_script_exe + ' ' + check_install_path + ' -l ' + index + ' -p ' + install_path
            code2 = functions.system_call_functions.system_call(cmd_check_path, message=False, returned=False)

            if (code2):
                check_install_module('1', index, '0', 'Install path package')
            else:
                check_install_module('0', index, '1', 'Install path package')
                print ("Please install module %s manually to continue with XICRA" %index)
                
        
################
## Miscellaneous
################
def print_module_comparison(module_name, message, color, tag):
    """
    Creates print message for a given module, version, message

    :param module_name: Name of the module
    :param message: Message to include in the print message: OK | FAILED | NOT FOUND
    :param color: Print message color: green | orange | red
    :param tag: Tag to include: Module, package, software

    :type module_name: string
    :type message: string 
    :type color: string
    :type tag: string

    :returns: Print message
    """
    print (colored("{:.<15}{:.>15}".format("%s: %s" %(tag, module_name), "[ %s ]" %message), color))

#########

##################

def check_install_module(installed, module_name, min_version, tag):
    """
    Checks modules installation 

    Checks whether a module is installed and fulfilling requirements.     
    It prints messages using :func:`XICRA.config.set_config.print_module_comparison`.

    :param installed: Version string of the module installed.
    :param module_name: Module name
    :param min_version: Version string for the minimum version required

    :type installed: string
    :type module_name: string 
    :type min_version: string
    """
     ## Not installed
    if (installed == 'n.a.' or not installed):
        message = 'NOT FOUND'
        color = 'red'
        print_module_comparison(module_name, message, color, tag)

    # check version
    elif LooseVersion(installed) >= LooseVersion(min_version):
        message = 'OK'
        color = 'green'
        print_module_comparison(module_name, message, color, tag)

    # check version
    elif (installed == 'ok'):
        message = 'OK'
        color = 'green'
        print_module_comparison(module_name, message, color, tag)

    else:
        message = 'FAILED'
        color = 'yellow'
        print_module_comparison(module_name, message, color, tag)

    ## return message
    return (message)

#########    



#!/usr/bin/ python3
##########################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain        ##
##########################################################
"""
Merge files from different sequencing lanes
"""

import pandas as pd
from termcolor import colored
import shutil
import concurrent.futures

from HCGB import functions

###############
def gunzip_merge(outfile, list_files):
    """
    Merge gunzip files into final file
    
    :param outfile: String for output file
    :param list_files: List of files to merge
    
    :type outfile: string
    :type list_files: list
        
    """
    list_files = list(list_files)
    list_files.sort()
    print ("\tMerging files into: ", outfile)
    #print ("\tFiles: ", list_files)

    with open(outfile, 'wb') as wfp:
        for fn in list_files:
            with open(fn, 'rb') as rfp:
                shutil.copyfileobj(rfp, wfp)

    return()
    
###############    
def one_file_per_sample(dataFrame, outdir_dict, threads, outdir, Debug=False):
    """
    Merge fastq files from different lanes positions for each sample
    
    """
    ## merge sequencing files for sample, no matter of sector or lane generated.    
    list_samples = set(dataFrame['new_name'].tolist())
    print (colored("\t" + str(len(list_samples)) + " samples to be merged from the input provided...", 'yellow'))
    print ("+ Merging sequencing files for samples")

    ##
    sample_frame = dataFrame.groupby(["new_name", "read_pair"])
    
    ### get extension for files
    ext_list = dataFrame.ext.unique()
    gz_list = dataFrame.gz.unique()
    ## might generate a bug if several extension or some zip/unzip files provided
    if gz_list:
        ext = ext_list[0] + gz_list[0]
    else:
        ext = ext_list[0]

    # We can use a with statement to ensure threads are cleaned up promptly
    with concurrent.futures.ThreadPoolExecutor(max_workers=threads) as executor: ## need to do 1 by one as there is a problem with the working directory
        commandsSent = { executor.submit(gunzip_merge, 
                                        outdir_dict[name[0]] + '/' + name[0] + '_' + name[1] + '.' + ext, 
                                        sorted(set(cluster["sample"].tolist()))): name for name, cluster in sample_frame }
        for cmd2 in concurrent.futures.as_completed(commandsSent):
            details = commandsSent[cmd2]
            try:
                data = cmd2.result()
            except Exception as exc:
                print ('***ERROR:')
                print (cmd2)
                print('%r generated an exception: %s' % (details, exc))                
                            
    ## return output name merged generated in dataframe
    name_columns = ("new_name", "dirname", "read_pair", "new_file", "ext", "gz")
    name_frame = pd.DataFrame(columns=name_columns)

    ## print to a file
    timestamp = functions.time_functions.create_human_timestamp()
    merge_details = outdir + '/' + timestamp + '_prep_mergeDetails.txt'
    merge_details_hd = open(merge_details, 'w')

    for name, cluster in sample_frame: ## loop over samples
        outfile = outdir_dict[name[0]] + '/' + name[0] + '_' + name[1] + ext
        
        merge_details_hd.write("####################\n")        
        merge_details_hd.write("Sample: " + name[0] + '\n')
        merge_details_hd.write("New name: " + name[0] + '\n')
        
        merge_details_hd.write("Read: " + name[1] + '\n')
        merge_details_hd.write("Files:\n")
        merge_details_hd.write(",".join(sorted(cluster["sample"].tolist())))
        merge_details_hd.write('\n')
        merge_details_hd.write("####################\n")        
        
        name_frame.loc[len(name_frame)] = (name[0], outdir_dict[name[0]], name[1], outfile, ext_list[0], gz_list[0])

    merge_details_hd.close()
    return(name_frame)



#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Maths functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
"""
## useful imports
import re

#####################
def percentage(percent, whole):
    if isinstance(percent, str):    
        if (percent == "0.00%"):
            return 0

        percent_search = re.search(r"(.*)%", percent)
        percent_int=0
        if percent_search:
            percent_int = float(percent_search.group(1))
        else:
            percent_int = percent
        value = (percent_int / whole) * 100

    elif isinstance(percent, int):
        value = (percent / whole) * 100
    
    return value


#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################

import json
import os
from HCGB import functions

######################################
def dump_info_run(out_folder, module, userinput, runInfo, debug):
    """
    Prints information of the job run in json format
    
    :param out_folder: Absolute path to folder to dump results
    :param module: Module or script name executed
    :param runInfo: Contains configuration detail information to dump
    
    :type out_folder: string 
    :type module: string 
    :type runInfo: dict
    
    :returns: Prints information in json format in the output folder provided.
    
    Example of information to include:
    
    userinput = {"filename":infiles, 
                "database":path_to_database,
                "file_format":file_format}
                
    runInfo = { "module":module, 
                "analysis":example,
                "date":date, 
                "time":time}
    
    Original idea extracted from https://bitbucket.org/genomicepidemiology/kmerfinder/src/master/kmerfinder.py
    
    """
    
    # Collect all data of run-analysis
    
    ## convert to dictionary
    userinput_dict = vars(userinput)
    del userinput_dict['func']
    
    ## merge dictionaries:: data = {**runInfo, **userinput_dict}
    
    ## add sub dictionary
    data = runInfo.copy()
    data['input_options'] = userinput_dict
    
    ## debug messages
    if debug:
        functions.aesthetics_functions.debug_message("Dump information:", 'yellow')
        print()
        functions.aesthetics_functions.debug_message("runInfo:", 'yellow')
        print(runInfo)
        print(type(runInfo))
        print()
        
        functions.aesthetics_functions.debug_message("userinput_dict:", 'yellow')
        print(userinput_dict)
        print(type(userinput_dict))
        print()
        
        functions.aesthetics_functions.debug_message("data:", 'yellow')
        print(data)
        print(type(data))
        print()
    
    # Save json output
    result_json_file = os.path.join(out_folder, module + ".json") 
    with open(result_json_file, "w") as outfile:  
        json.dump(data, outfile, indent=3)

######################################
def compare_info_dict(dict1, dict2, listItems, debug):
    """Given a pair of dictionaries and a list of items, compare values."""
    print()
    
    
    


#!/usr/bin/env python3
import setuptools

#######
def get_require_modules(given_file):
    """
    Get main python requirements modules
    """
    with open(given_file, 'r') as f:
        myModules = [line.strip().split(',')[0] for line in f]
    
    return myModules

#######
def get_version(version_file):
    """
    Original code: PhiSpy setup.py 
    https://github.com/linsalrob/PhiSpy/blob/master/setup.py
    """
    with open(version_file, 'r') as f:
        v = f.readline().strip()

    return v




long_description_text = ""
with open("README.md", "r") as fh:
    long_description_text = fh.read()

setuptools.setup(
    name="HCGB",
    version=get_version("./VERSION"),

    author="Jose F. Sanchez-Herrero",
    description="Useful python functions",

    author_email="jfbioinformatics@gmail.com",

    long_description_content_type="text/markdown",
    long_description=long_description_text,

    url="https://github.com/HCGB-IGTP/HCGB_python_functions/",

    packages=setuptools.find_packages(),
    license='MIT License',

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    install_requires=get_require_modules("./HCGB/config/python_requirements.txt"),
)



#!/usr/bin/env python3
#############################################################
## Jose F. Sanchez, Marta Lopez & Lauro Sumoy              ##
## Copyright (C):2019-2021 Lauro Sumoy Lab, IGTP, Spain    ##
#############################################################
"""
split_GTF converts GTF file into many small GTF files.
Usage: split_GTF.py {OPTIONS} [.GTF file]

It splits GTF into given number of files. It takes into account no genes or transcript are broken.
It is also possible to split according to chromosome (one gtf/chromosome)
"""

import os
import re
import argparse
import traceback
from termcolor import colored

import HCGB.functions.aesthetics_functions as HCGB_aes
import HCGB.functions.time_functions as HCGB_time
import HCGB.functions.files_functions as HCGB_files

############################################################
def create_names(file2split, name_file, chr_option, num_files, in_format, debug=False):
    """
    Creates names for subsets of files generated.
    """
    ##
    dict_files_generated = {}
    
    if debug:
        print()
        HCGB_aes.debug_message("********************** create_names ********************** ", "yellow")
        HCGB_aes.debug_message("Create name for file subsets", "yellow")
    
    ########################
    ## Chromosome option
    ########################
    if chr_option:
        
        print(file2split)
        
        ## get list of entries
        with open(file2split) as f:
            
            list2 = []
            for row in f:
                row = row.rstrip() ## remove empty line
                if row:
                    item_chr = row.split()[0]
                    list2.append(item_chr)
                
            ## get uniq
            list2  = set(list2)
            
            ## remove # characters
            list2 = [x for x in list2 if not x.startswith('#')]
                  
        ## create files for list of entries
        for seq in list2:
            file_name = name_file + "-Chr_" + str(seq) + "." + in_format.lower()
            dict_files_generated["Chr_" + str(seq)] = file_name

    else:
        ########################
        ## Number files to create
        ########################
        if debug:
            HCGB_aes.debug_message("num_files: " + str(num_files), "yellow")

        for fileCount in range(num_files):
            file_name = name_file + "-" + str(fileCount+1) + "." + in_format.lower()
            dict_files_generated["File_" + str(fileCount+1)] = file_name

    if debug:
        print()
        HCGB_aes.debug_message("dict_files_generated: " + str(dict_files_generated), "yellow")
        print()
        HCGB_aes.debug_message("--------------------- create_names_GTF --------------------- ", "yellow")
        print()

    ## return
    return dict_files_generated

############################################################
def split_file_call(given_file, num_files, name, chr_option, in_format, path_given=False, debug=False):
    """
    This functions checks if it has been done previously the split of file. 
    If done, returns dict with files names generated using create_names.
    If split not done, it calls split_file function. 
    
    :param given_file: Absolute path to the file to split
    :param num_files: Number of files to create with equal number of lines
    :param name: Name to include in the files names generated. By default, file basename included.
    :param chr_option: TRUE/FALSE If chr_option provided, split file into chromosome, scaffolds or reference sequences.
    :param path_given: Path to save results. Default use absolute path of file provided
    :param debug: TRUE/FALSE for debugging messages
    """

    ## debug messaging    
    if debug:
        print()
        HCGB_aes.debug_message("********************** split_file_call ********************** ")
        HCGB_aes.debug_message("Checking if file has been previously splitted", "yellow")
    

    print("+ Splitting file provided: " + given_file)
    
    if path_given:
        path_given = os.path.abspath(path_given)
        HCGB_files.create_folder(path_given)
        print("+ Output: " + path_given)
    else:
        path_given = os.getcwd()
        print("+ Output: Current working directory: " + path_given )
    
    ## get absolute path and name
    name_file = HCGB_files.get_path_name(given_file, path_given, name, debug=debug)

    print("+ Checking if previously done...")

    filename_stamp = path_given + '/.split_file_success'
    if os.path.isfile(filename_stamp):
        
        if debug:
            HCGB_aes.debug_message("Time stamp exists: .split_file_success ", "yellow")

        ## check file names generated and return names
        files_generated = create_names(given_file, name_file, chr_option, num_files, in_format, debug=debug)

        re_run=False
        for f in files_generated.values():
            if not HCGB_files.is_non_zero_file(f):
                re_run=True
                
                if debug:
                    print()
                    HCGB_aes.debug_message("It is required to re-run split: File is zero or does not exists\n" + f, "yellow")
                
                break
        if not re_run:
            stamp = HCGB_time.read_time_stamp(filename_stamp)
            print("")
            print (colored("\tA previous command generated results on: %s [%s]" %(stamp, 'split file'), 'yellow'))
            return (files_generated)

    print("+ Not previously done or some error ocurred during the process")
    print()
    print("+ Let's do it now!")

    ## call to split 
    files_generated = split_file(given_file, num_files, name_file, chr_option, in_format, path_given, debug)
    
    ## print time stamp
    HCGB_time.print_time_stamp(filename_stamp)
    
    if debug:
        HCGB_aes.debug_message("********************** split_file_call ********************** ")
    
    
    return (files_generated)    
    
############################################################
def split_file(given_file, num_files, name, chr_option, in_format, path_given=False, debug=False):
    """
    This functions splits given file (GTF or BED) into multiple files, either a given number of files or
    one for each chromosome.

    :param given_file: Absolute path to file to split
    :param num_files: Number of files to create with equal number of lines
    :param name: Name to include in the files names generated. By default, file basename included.
    :param chr_option: TRUE/FALSE If chr_option provided, split into chromosome, scaffolds or reference sequences.
    :param path_given: Path to save results. Default use absolute path of file provided
    :param debug: TRUE/FALSE for debugging messages
    """
    ## init dict to store files generated
    dict_files_generated = create_names(given_file, name, chr_option, num_files, in_format, debug=debug)

    if debug:
        print()
        HCGB_aes.debug_message("*************************** split_GTF ***************************", "yellow")
        HCGB_aes.debug_message("given_file: " + given_file, "yellow")
        HCGB_aes.debug_message("num_files: " + str(num_files), "yellow")        
        HCGB_aes.debug_message("chr_option: " + str(chr), "yellow")
        HCGB_aes.debug_message("name: " + name, "yellow")
        HCGB_aes.debug_message("path_given: " + path_given, "yellow")

    ## Check file is readable
    if not (HCGB_files.is_non_zero_file(given_file)):
        print("ERROR: File not readable. Please check path for file:\n" + given_file)
        exit()

    ## get absolute path and name
    #name = HCGB_files.get_path_name(given_file, path_given, name, debug=debug)

    if debug:
        HCGB_aes.debug_message("name: " + name, "yellow")
    
    print("")
    
    #read a file
    fileReader = open(given_file)
            
        
    ## Get options
    if (chr_option):
        #######################################################
        ### Split file by Chromosome: Chr option
        #######################################################
        ## This option is the same for GTF and BED files.
        ## Chromosome or reference sequence is the first field always.

        ## Prevalence of chr split if provided.
        print("+ Splitting file by reference sequence...")
        
        try:
            ## skip comments at the beginning of files
            while True:
                line = fileReader.readline()
                if not line.startswith('#'):
                    break
            
            lineCount=0
            line = line.rstrip()
            field=line.strip().split('\t')
            chrid=field[0]
             
            stop=False
            line2=""
    
            while True: 
                
                # empty line
                line = line.rstrip()
                if line:
                    field=line.strip().split('\t')
                    chrid=field[0]
                
                    ## create new file
                    if lineCount == 0:
                        #create a file in append mode
                        ## open file
                        fileWriter = open(dict_files_generated["Chr_" + str(chrid)],"a") ## append as it might be some repetitive elements at the end                
        
                    #write a line
                    fileWriter.write(line + '\n')
                    
                    ## Debug messages
                    if debug:
                        HCGB_aes.debug_message("Chr: " + str(chrid), "red")
                        HCGB_aes.debug_message("line: " + line, "red")
                    
                    ## stop when Chr changes
                    while True:
                        ## read new line
                        line2 = fileReader.readline()
                        line2 = line2.rstrip()

                        field2=line2.strip().split('\t')
                        chrid2=field2[0]
        
                        ## debug messages    
                        if debug:
                            HCGB_aes.debug_message("line: " + line, "red")
                            HCGB_aes.debug_message("line2: " + line2, "red")
                            HCGB_aes.debug_message("geneid: " + chrid, "yellow")
                            HCGB_aes.debug_message("chrid2: " + chrid2, "yellow")
        
                        ##
                        if (chrid == chrid2):
                            fileWriter.write(line2 + '\n')
                            line=line2
                            
                        else:
                            ## init
                            lineCount = 0
                            fileWriter.close()
                            line=line2
                            break
                else:
                    break
    
            ## Close GTF File        
            fileWriter.close()
        
        except Exception as e:
            #print the exception if any
            print(e.__traceback__)
            traceback.print_exc()
        finally:
            #close the file reader
            fileReader.close()

    else:
        
        print("+ Splitting file into a given number of files... " + str(num_files) + ' files requested')

        ###############################################3
        ## Split into several files as provided.
        ###############################################3
        lineCount = 0
        fileCount = 1

        # max lines you want to write in a single file
        totalLines_file = sum(1 for line in open(given_file))
        fileLineCount = int(totalLines_file/num_files)+1

        ## Debug messages
        if debug:
            HCGB_aes.debug_message("totalLines_file: " + str(totalLines_file), "yellow")
            HCGB_aes.debug_message("fileLineCount: " + str(fileLineCount), "yellow")
            HCGB_aes.debug_message("num_files: " + str(num_files), "yellow")
                
        try:
            #read a file
            ## skip comments at the beginning of files
            while True:
                line = fileReader.readline()
                if not line.startswith('#'):
                    break
    
            stop=False
            line2=""
            
            line = line.rstrip()
            
            while True:
                # empty line
                line = line.rstrip()
                if line:
                    field=line.strip().split('\t')

                    ## create new file
                    if lineCount == 0:
                        #create a file in write mode
                        fileWriter = open(dict_files_generated["File_" + str(fileCount)],"w")
                        #increment file count, use it for new file name
                        fileCount += 1
    
                    ## Debug messages
                    #if debug:
                    #    HCGB_aes.debug_message("lineCount: " + str(lineCount), "red")
                    #    HCGB_aes.debug_message("line: " + line, "red")
                    
                    ## stop when max_lines_file achieved
                    if lineCount == fileLineCount:
                        
                        if in_format=="GTF":
                            ## If GTF file provided, control we are not splitting genes
                            while True:
                                field=line.strip().split('\t')
                                geneid=re.findall(r'gene_id \"([\w\.]+)\"',field[8])
        
                                ## read new line
                                line2 = fileReader.readline()
                                lineCount2 = lineCount + 1
                                field2=line2.strip().split('\t')
                                geneid2=re.findall(r'gene_id \"([\w\.]+)\"',field2[8])
        
                                ## debug messages    
                                #if debug:
                                #    HCGB_aes.debug_message("lineCount: " + str(lineCount), "red")
                                #    HCGB_aes.debug_message("line: " + line, "red")
                                #    HCGB_aes.debug_message("lineCount2: " + str(lineCount2), "red")
                                #    HCGB_aes.debug_message("line2: " + line2, "red")
                                #    HCGB_aes.debug_message("geneid: " + geneid[0], "yellow")
                                #    HCGB_aes.debug_message("geneid2: " + geneid2[0], "yellow")
                                
                                ##
                                if (geneid[0] == geneid2[0]):
                                    fileWriter.write(line2 + '\n')
                                else:
                                    stop=True
                                    break
        
                                ## init
                                lineCount = 0
                                fileWriter.close()
                            
                            if not stop:
                                #read a line
                                line = fileReader.readline()
                                if line == '':#empty is EOF
                                    fileWriter.close()
                            else:
                                line=line2
                                stop=False
                            
                        ## In BED format there is no problem of breaking exons, etc
                        elif in_format=="BED" or in_format=="SAM":
                            ## init
                            lineCount = 0
                            fileWriter.close()
                    else:
                        ## sum to lines
                        lineCount += 1
    
                        #write a line
                        fileWriter.write(line + '\n')

                        line = fileReader.readline()
                        if line == '':#empty is EOF
                            fileWriter.close()
                else:
                    break
                
            ## Close GTF File        
            fileWriter.close()
        
        except Exception as e:
            #print the exception if any
            print(e.__traceback__)
            traceback.print_exc()
        finally:
            #close the file reader
            fileReader.close()

    ##
    if debug:
        print()
        HCGB_aes.debug_message("dict_files_generated: " + str(dict_files_generated), "yellow")
        print()
        HCGB_aes.debug_message("*************************** split_GTF ***************************", "yellow")
    
    print("\t\t Process finished here.")
    
    return(dict_files_generated)

############################################################
def main():
    ## this code runs when call as a single script
    parser=argparse.ArgumentParser(description='''Split GTF file into multiple files
    Note:
    It takes into account no genes or transcript are broken
    It is also possible to split according to chromosome (one gtf/chromosome)
    
    ''');
    
    parser.add_argument('--input', '-i', help='Input file', required=True);
    
    parser.add_argument('--input_format', '-f', dest='in_format', nargs='*', help='Input format file', choices=['BED', 'GTF'], required=True);
    
    parser.add_argument('--num_files','-n', type=int,
                        help='Split file into as many subfiles.', default=2);
    
    parser.add_argument('--name',
                        help='Name to add for each file generated. Default: use filename provided.', default="");

    parser.add_argument('--path',
                        help='Path to save for each file generated. Default: use path from filename provided.', 
                        default="");

    parser.add_argument('--split_chromosome','-c',action="store_true",
                        help='Split file for each chromosome or reference sequence available.');

    args=parser.parse_args();
    
    ## lets split the big file provided
    files_generated = split_file_call(os.path.abspath(args.input), num_files=args.num_files, name=args.name, 
                  chr_option=args.split_chromosome, in_format=str(args.in_format[0]), 
                  path_given=os.path.abspath(args.path), 
                  debug=False)
    
    print("+ Check dictionary with files generated:")    
    print(files_generated)
    

############################################################
if __name__== "__main__":
    main()


__all__ = [
    'aesthetics_functions',
    'blast_functions',
    'fasta_functions',
    'files_functions',
    'main_functions',
    'system_call_functions',
    'math_functions',
    'info_functions',
    'time_functions'
    
]

from HCGB.functions import *




#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Shared functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
With different purposes:
    - Print time stamps
  
    - Create system calls

    - Manage/list/arrange files/folders

    - Aesthetics

    - Manage fasta files

    - Other miscellaneous functions
"""
## useful imports
import sys
from termcolor import colored

############################################################################
########                     AESTHETICS                             ########
############################################################################
def pipeline_header(option, ver=""):
    """
    Prints a common header for the pipeline including name, author, copyright and year.        
    """
    
    print ("\n")
    print_sepLine("#", 70, False)
    
    if (option == 'XICRA'):
        print('#', '{: ^66}'.format("XICRA pipeline"), '#')
        print('#', '{: ^66}'.format("Jose F. Sanchez & Lauro Sumoy"), '#')
    elif (option == 'BacterialTyper'):
        print('#', '{: ^66}'.format("BacterialTyper pipeline"), '#')
        print('#', '{: ^66}'.format("Jose F. Sanchez, Cristina Prat & Lauro Sumoy"), '#')
    
    print('#', '{: ^66}'.format("Copyright (C) 2019-2021 Lauro Sumoy Lab, IGTP, Spain"), '#')
    
    ## add version
    if ver:
        print('#', '{: ^66}'.format("Version: " + str(ver)), '#')
    
    print_sepLine("#", 70, False)

###############
def print_sepLine(char, num, color):
    string = char * num
    if (color):
        print (colored(string, color))
    else:
        print (string)

###############
def boxymcboxface(message):
    ## this function is from ARIBA (https://github.com/sanger-pathogens/ariba)
    ## give credit to them appropiately
    #print('-' * 79)
    print ('\n')
    print('|', '=' * 50, '|', sep='')
    print('|', '{: ^48}'.format(message), '|')
    print('|', '=' * 50, '|', sep='')
    print ('\n')
    #print('-' * 79)

###############
def progbar(curr, total, full_progbar):
    frac = (curr/total)
    filled_progbar = round(frac*full_progbar)
    print ('\r', '#'*filled_progbar + '-'*(full_progbar-filled_progbar), '[{:>7.2%}]'.format(frac), end='')
    sys.stdout.flush()
    
###############
def debug_message(message, color='red'):
    print (colored("*** DEBUG: %s" %(message), color))   

